# February 2022 - ESM, Rollup, and Vitest

The initial project setup followed [oclif](https://github.com/oclif/oclif) recommendation:
*CommonJS-based Javascript generated by the Typescript compiler.*
The dev executable generated by oclif would run the CLI through [ts-node](https://github.com/TypeStrong/ts-node).
That worked fine until we started adding some pure [ESM](https://hacks.mozilla.org/2018/03/es-modules-a-cartoon-deep-dive/) dependencies.
They cause the setup to break because the [interoperability is only from ESM to CommonJS](https://nodejs.org/api/esm.html#interoperability-with-commonjs).
In other words, ESM projects can import CommonJS code, but not the other way around.
It wouldn't be an issue if we used only CommonJS dependencies,
but considering more dependencies are migrating to ESM,
sticking with CommonJS might put us in a position where we won't be able to use some dependencies that are necessary for the CLI.
Motivated by **aligning the CLI with the ecosystem's future** and **ensuring that we can use CommonJS and ESM dependencies**,
we [turned the project into an ESM project](https://github.com/Shopify/shopify-cli-next/pull/16).
Moreover, we took the opportunity to introduce [Rollup](https://rollupjs.org/guide/en/) as a build tool to make the CLI more lightweight and minimize the dependencies that need to be resolved and installed on the user side.
Resolving and installing deep dependency graphs might lead to graph with incompabilities between nodes *(i.e. the well-known delete `node_modules` to fix the problem solution)*.

**`@shopify/cli-kit`** [builds](https://github.com/Shopify/shopify-cli-next/pull/16/files#diff-2f784764604720e8134392c8d17d6a3f1097255ce2243fc1c5ef16cf9f39452a) a bundle under `dist/index.js` that includes the external dependencies. Internal teams and external developers building and extending the CLI can thus make [`@shopify/cli-kit`](/kit/introduction) a `peerDependency` of the project and have access to a set of tools to build an experience that's consistent with the rest of the CLI.

**`@shopify/create-app`** [builds](https://github.com/Shopify/shopify-cli-next/pull/16/files#diff-849e531f8f8d0a646bee358c606d2f9c674231ea6dcba6232b9a1946bbbe4754) a bundle under `dist/commands/init.js` that includes external dependencies including `@shopify/cli-kit`. We bundle `@shopify/cli-kit` instead of treating it as an external dependency that needs to be installed when creating apps to let Rollup [tree-shake](https://rollupjs.org/guide/en/#tree-shaking) it and only include the elements that are necessary for creating apps.

**`@shopify/cli`** [builds](https://github.com/Shopify/shopify-cli-next/pull/16/files#diff-15aa6fca366d53af665e12ea56de980a410691fca346464ad75fbd4264465113) a bundle under `dist/commands/*.js` representing the commands from `@shopify/app` and `@shopify/theme`. Unlike `@shopify/create-app`, `@shopify/cli-kit` is a `dependency` so that external plugins that add additional functionality to the CLI can depend on it.

While working on enabling this setup, we encountered some Jest limitations because the framework's support for ESM is [still experimental](https://github.com/facebook/jest/issues/9430). In particular, the mocking functionality that we use of from the CLI doesn't work.
To remove the blocker while the full support is being worked on,
we set up a different test framework,
[Vitest](https://vitest.dev/),
which was designed for ESM and has an API [compatible with Jest's](https://vitest.dev/guide/features.html#chai-and-jest-expect-compatibility).


### Resources

- [Hello, Modules!](https://blog.sindresorhus.com/hello-modules-d1010b4e777b)
- [Pure ESM Package](https://gist.github.com/sindresorhus/a39789f98801d908bbc7ff3ecc99d99c)
